<?php

use CCR\MailWrapper;

// Operation: mailer->contact

$response = array();

\xd_security\assertParametersSet(array(
  'name'              => RESTRICTION_FIRST_NAME,
  'message'           => RESTRICTION_NON_EMPTY,
  'username'          => RESTRICTION_NON_EMPTY,
  'token'             => RESTRICTION_NON_EMPTY,
  'timestamp'         => RESTRICTION_NON_EMPTY
));

\xd_security\assertEmailParameterSet('email');

// ----------------------------------------------------------

// If the user claims to be a public user, verify that they're not logged in.
// Otherwise, verify that their claimed username matches the logged in user.
$user_is_public = $_POST['username'] === '__public__';

if ($user_is_public) {
  $user_logged_in = false;
  try {
    \xd_security\getLoggedInUser();
    $user_logged_in = true;
  } catch (Exception $e) {}

  if ($user_logged_in) {
    \xd_response\presentError("Client claims to be public user but is logged in.");
  }
} else {
  $user = \xd_security\getLoggedInUser();
  if ($_POST['username'] !== $user->getUsername()) {
    \xd_response\presentError("Client claims to be a user other than the logged in user.");
  }
}

// ----------------------------------------------------------

$user_info = $user_is_public ? 'Public Visitor' : "Username:     ".$_POST['username'];
$reason = isset($_POST['reason']) ? $_POST['reason'] : 'contact';

// ----------------------------------------------------------

\xd_utilities\verify_captcha();

// ----------------------------------------------------------

switch ($reason) {
  case 'wishlist':
    $subject = "[WISHLIST] Feature request sent from a portal visitor";
    $message_type = "feature request";
    break;

  default:
    $subject = "Message sent from a portal visitor";
    $message_type = "message";
    break;
}

$timestamp = date('m/d/Y, g:i:s A', $_POST['timestamp']);

$message = "Below is a $message_type from '{$_POST['name']}' ({$_POST['email']}):\n\n";
$message .= $_POST['message'];
$message .="\n------------------------\n\nSession Tracking Data:\n\n  ";
$message .="$user_info\n\n  Token:        {$_POST['token']}\n  Timestamp:    $timestamp";

try {
    //Original sender's e-mail must be in the 'fromAddress' field for the XDMoD Request Tracker to function
    MailWrapper::sendMail(array(
        'body'         => $message,
        'subject'      => $subject,
        'toAddress'    => \xd_utilities\getConfiguration('general', 'contact_page_recipient'),
        'fromAddress'  => $_POST['email'],
        'fromName'     => $_POST['name']
        )
    );
}
catch (Exception $e) {
    $response['success'] = false;
    $response['message'] = $e->getMessage();
    echo json_encode($response);
    exit;
}

// =====================================================

$message
    = "Hello, {$_POST['name']}\n\n"
    . "This e-mail is to inform you that the XDMoD Portal Team has received your $message_type, and will\n"
    . "be in touch with you as soon as possible.\n\n"
    . MailWrapper::getMaintainerSignature();

// -------------------

try {
    MailWrapper::sendMail(array(
        'body'      => $message,
        'subject'   => "Thank you for your $message_type.",
        'toAddress' => $_POST['email']
        )
    );
}
catch (Exception $e) {
    $response['success'] = false;
    $response['message'] = $e->getMessage();
    echo json_encode($response);
    exit;
}

// =====================================================

$response['success'] = true;

echo json_encode($response);
